import socket
import threading
import tkinter as tk
from tkinter import scrolledtext

# Configuración del cliente
SERVER_HOST = 'servidor'  # Dirección del servidor (nombre del contenedor Docker o IP)
SERVER_PORT = 65432       # Puerto del servidor
CLIENT_NAME = "cliente1"  # Nombre único del cliente (puedes cambiarlo según el contenedor)

# Función para recibir mensajes del servidor
def recibir_mensajes(sock, text_area):
    while True:
        try:
            mensaje = sock.recv(1024).decode()
            if mensaje:
                text_area.config(state=tk.NORMAL)
                text_area.insert(tk.END, mensaje + "\n")
                text_area.config(state=tk.DISABLED)
                text_area.see(tk.END)
            else:
                break
        except ConnectionResetError:
            text_area.config(state=tk.NORMAL)
            text_area.insert(tk.END, "Conexión cerrada por el servidor.\n")
            text_area.config(state=tk.DISABLED)
            break

# Función para enviar mensajes al servidor
def enviar_mensaje(sock, entrada_texto, text_area):
    mensaje = entrada_texto.get().strip()
    if mensaje:
        try:
            sock.sendall(mensaje.encode('utf-8'))
            entrada_texto.delete(0, tk.END)

            # Mostrar el mensaje enviado en el área de texto
            text_area.config(state=tk.NORMAL)
            text_area.insert(tk.END, f"Tú: {mensaje}\n")
            text_area.config(state=tk.DISABLED)
            text_area.see(tk.END)
        except Exception as e:
            text_area.config(state=tk.NORMAL)
            text_area.insert(tk.END, f"Error al enviar el mensaje: {e}\n")
            text_area.config(state=tk.DISABLED)

# Función para cerrar la conexión y la ventana
def cerrar_cliente(sock, root):
    try:
        sock.close()  # Cerrar la conexión con el servidor
    except Exception as e:
        print(f"Error al cerrar la conexión: {e}")
    root.destroy()  # Cerrar la ventana

# Crear la ventana gráfica del cliente
def iniciar_cliente():
    root = tk.Tk()
    root.title(f"Cliente: {CLIENT_NAME}")

    # Área de texto para mostrar mensajes
    text_area = scrolledtext.ScrolledText(root, width=40, height=20)
    text_area.pack(padx=10, pady=10)
    text_area.config(state=tk.DISABLED)

    # Entrada de texto para enviar mensajes
    entrada_texto = tk.Entry(root, width=40)
    entrada_texto.pack(padx=10, pady=5)

    # Conexión al servidor
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect((SERVER_HOST, SERVER_PORT))
        sock.sendall(CLIENT_NAME.encode('utf-8'))  # Enviar el nombre del cliente al servidor

        # Mostrar conexión exitosa en el área de texto
        text_area.config(state=tk.NORMAL)
        text_area.insert(tk.END, f"Conectado al servidor {SERVER_HOST}:{SERVER_PORT}\n")
        text_area.config(state=tk.DISABLED)
    except Exception as e:
        text_area.config(state=tk.NORMAL)
        text_area.insert(tk.END, f"Error al conectar al servidor: {e}\n")
        text_area.config(state=tk.DISABLED)
        return

    # Hilo para recibir mensajes
    threading.Thread(target=recibir_mensajes, args=(sock, text_area), daemon=True).start()

    # Contenedor para los botones
    boton_frame = tk.Frame(root)
    boton_frame.pack(pady=5)

    # Botón para enviar mensajes
    boton_enviar = tk.Button(boton_frame, text="Enviar", command=lambda: enviar_mensaje(sock, entrada_texto, text_area))
    boton_enviar.pack(side=tk.LEFT, padx=5)

    # Botón para cerrar la ventana
    boton_cerrar = tk.Button(boton_frame, text="Cerrar", command=lambda: cerrar_cliente(sock, root))
    boton_cerrar.pack(side=tk.LEFT, padx=5)

    root.protocol("WM_DELETE_WINDOW", lambda: cerrar_cliente(sock, root))  # Manejar el cierre de la ventana
    root.mainloop()

if __name__ == "__main__":
    iniciar_cliente()