import socket
import threading

# Configuración del servidor
SERVER_HOST = '0.0.0.0'
SERVER_PORT = 65432
CLIENTS = {}  # Diccionario para almacenar conexiones y nombres de clientes

def manejar_cliente(conn, addr):
    try:
        # Recibir el nombre del cliente
        client_name = conn.recv(1024).decode()
        CLIENTS[conn] = client_name
        print(f"{client_name} conectado desde {addr}")

        while True:
            mensaje = conn.recv(1024).decode()
            if mensaje:
                print(f"{client_name}: {mensaje}")
                # Reenviar el mensaje a otros clientes
                for client_conn, name in CLIENTS.items():
                    if client_conn != conn:
                        client_conn.sendall(f"{client_name}: {mensaje}".encode())
            else:
                break
    except ConnectionResetError:
        pass
    finally:
        # El cliente se ha desconectado
        print(f"{CLIENTS[conn]} se ha desconectado.")
        del CLIENTS[conn]
        conn.close()

def iniciar_servidor():
    # Crear socket
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind((SERVER_HOST, SERVER_PORT))
    server_socket.listen(5)
    print(f"Servidor escuchando en {SERVER_HOST}:{SERVER_PORT}")

    while True:
        conn, addr = server_socket.accept()
        threading.Thread(target=manejar_cliente, args=(conn, addr), daemon=True).start()

if __name__ == "__main__":
    iniciar_servidor()